/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Demo
 *
 *********************************************************************
 * FileName:        user_generic_hid.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 * Jan Axelson           3/4/05     Mouse application adapted for 
 *                                  Generic HID application.
 *                       3/30/05    Minor bug fixes and enhancements.  		
 ********************************************************************/

/*

This application is a generic HID device (not a system keyboard or mouse) 
that does the following:

Receives Output reports using interrupt OUT transfers and sends the report 
data back to the host in Input reports using interrupt IN transfers.

Receives Output reports using control transfers (Set_Report request)
and sends the report data back to the host in Input reports using control 
transfers (Get_Report requests.)

The application uses separate buffers for Output reports received via 
interrupt transfers and Output reports received via control transfers use
separate buffers. The application uses separate buffers for Input reports 
received via interrupt transfers and Input reports received via control 
transfers.

Receives Feature reports using control transfers (Set_Report request)
and sends the report data back to the host in Feature reports using 
control transfers (Get_Report requests.)

A single buffer holds received Feature reports and reports to be sent.

This example is adapted from Microchip's Mouse example.

This application and host applications to communicate with the device are available
from www.Lvr.com/hidpage.htm

*/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user_generic_hid.h"

/** V A R I A B L E S ********************************************************/
#pragma udata

// BOOL emulate_mode;
// rom signed char dir_table[]={-4,-4,-4, 0, 4, 4, 4, 0};
// byte movement_length;
// byte vector = 0;
byte number_of_bytes_read;
char transmit_buffer[EP0_BUFF_SIZE];
char receive_buffer[EP0_BUFF_SIZE];

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
void ReportLoopback(void);
void HandleControlOutReport(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
    mInitAllLEDs();

}//end UserInit

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/

void ProcessIO(void)
{   
	// Called repeatedly in main().

    BlinkUSBStatus();

    // User Application USB tasks

    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

    ReportLoopback();
    
}//end ProcessIO

/******************************************************************************
 * Function:        void ReportLoopback(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        	Checks to see if an Output report has been received on 
						the Interrupt OUT endpoint. If yes, sends the report 
						data back as an Input report on the Interrupt IN endpoint.

						Shows how to extract data from an Output report received 
						on an interrupt endpoint and how to provide data to be 
						sent in an Input report on an interrupt endpoint.
 *
 * Note:            None
 *****************************************************************************/

void ReportLoopback(void)
{
	byte count;

	// Find out if an Output report has been received from the host.

	number_of_bytes_read = HIDRxReport(receive_buffer, HID_OUTPUT_REPORT_BYTES);

	if (number_of_bytes_read > 0)
	{
		// An Output report was received.

		// This example application copies the received report to 
		// transmit_buffer to send in the next Input report.

		for (count = 1; count <= HID_OUTPUT_REPORT_BYTES; count = count + 1)
		{				
			transmit_buffer[count-1] = receive_buffer[count-1];			
		}

		// If necessary, wait until the interrupt IN endpoint isn't busy.

		while(mHIDTxIsBusy())
		{
			// Service USB interrupts.

			USBDriverService(); 
		}

		// The report will be sent in the next interrupt IN transfer.

   	    HIDTxReport(transmit_buffer, HID_INPUT_REPORT_BYTES);
	}
}//end ReportLoopback

/******************************************************************************
 * Function:        void HandleControlOutReport(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Checks to see if an Output or Feature report has arrived
 *         			on the control pipe. If yes, extracts and uses the data.
 *
 * Note:            None
 *****************************************************************************/

void HandleControlOutReport()
{
	byte count;

	// Find out if an Output or Feature report has arrived on the control pipe.
	// Get the report type from the Setup packet.

	switch (MSB(SetupPkt.W_Value))
    {
		case 0x02: // Output report 

			// Get the report ID from the Setup packet.

    		switch(LSB(SetupPkt.W_Value))
		    {
				case 0: // Report ID 0

					// This example application copies the Output report data 
					// to hid_report_in. 
					// (Assumes Input and Output reports are the same length.)
					// A "real" application would do something more useful with the data.

				    // wCount holds the number of bytes read in the Data stage.
					// This example assumes the report fits in one transaction.	
				
					for (count = 1; count <= HID_OUTPUT_REPORT_BYTES; count = count + 1)
					{
						hid_report_in[count-1] = hid_report_out[count-1];
					}				
					// The number of bytes in the report (from usbcfg.h).
		
					break;		
			} // end switch(LSB(SetupPkt.W_Value))

		case 0x03: // Feature report 

			// Get the report ID from the Setup packet.

    		switch(LSB(SetupPkt.W_Value))
		    {
				case 0: // Report ID 0

				// The Feature report data is in hid_report_feature.
				// This example application just sends the data back in the next
				// Get_Report request for a Feature report.			
			
			    // wCount holds the number of bytes read in the Data stage.
				// This example assumes the report fits in one transaction.	
		
				break;
			} // end switch(LSB(SetupPkt.W_Value))		

	} // end switch(MSB(SetupPkt.W_Value))

} // end HandleControlOutReport

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/******************************************************************************
 * Function:        void GetInputReport0(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       	Prepares to send the data in hid_report_out in the Data 
 * 					stage of the current Get_Report transfer.
 *
 * Note:            None
 *****************************************************************************/

void GetInputReport0(void)
{
	byte count;

	// Set pSrc.bRam to point to the report.
	// 

	pSrc.bRam = (byte*)&hid_report_in;

	// The number of bytes in the report (from usbcfg.h).

	LSB(wCount) = HID_INPUT_REPORT_BYTES;

} // end GetInputReport0

/******************************************************************************
 * Function:        void GetFeatureReport0(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:       	Prepares to send the data in hid_report_feature in the Data 
 * 					stage of the current Get_Report transfer.
 *
 * Note:            None
 *****************************************************************************/

void GetFeatureReport0(void)
{
	byte count;

	// Set pSrc.bRam to point to the report.

	pSrc.bRam = (byte*)&hid_report_feature;

	// The number of bytes in the report (from usbcfg.h).

    LSB(wCount) = HID_FEATURE_REPORT_BYTES;

} // end GetFeatureReport0

/** EOF user_generic_hid.c *********************************************************/
